"""
Lightweight bootstrapping for project scripts
"""

from importlib.metadata import PackageNotFoundError, version

try:
    __version__ = version("sandal")
except PackageNotFoundError:
    # package is not installed
    pass



import os
import sys
sys.path.insert(0, os.path.abspath(".."))

import sandal

project = "Sandal"
copyright = "2023 Michael Ekstrand"
author = "Michael D. Ekstrand"

release = sandal.__version__

extensions = [
    "sphinx.ext.napoleon",
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.intersphinx",
    "sphinxext.opengraph",
]

source_suffix = ".rst"

pygments_style = "sphinx"
highlight_language = "python3"

html_theme = "furo"
html_theme_options = {
}
templates_path = ["_templates"]

intersphinx_mapping = {
    "python": ("https://docs.python.org/3/", None),
    "numpy": ("https://docs.scipy.org/doc/numpy/", None),
    "sklearn": ("https://scikit-learn.org/stable/", None),
    "progress_api": ("https://progress-api.readthedocs.io/en/latest/", None),
}

autodoc_default_options = {
    "members": True,
    "member-order": "bysource"
}
autodoc_typehints = "description"



"""
Setup routines for CLI environments.
"""

import logging
import sys

from .loghelper import Verbosity, vrb_to_level

_log_initialized = False

nb_fmt = logging.Formatter(
    "[%(asctime)s] %(levelname)-8s %(name)s %(message)s",
)


def setup_logging(verbose: Verbosity):
    """
    Set up logging for a notebook environment.

    Args:
        verbose: the verbosity level.
    """
    global _log_initialized
    level = vrb_to_level(verbose)
    root = logging.getLogger()

    if not _log_initialized:
        handler = logging.StreamHandler(stream=sys.stderr)
        handler.setFormatter(nb_fmt)
        root.addHandler(handler)
        _log_initialized = True

    root.setLevel(level)



"""
Setup routines for CLI environments.
"""

import logging
import sys
from pathlib import Path

import progress_api
from colorlog import ColoredFormatter
from enlighten import Manager
from progress_api.backends.enlighten import EnlightenProgressBackend

from .loghelper import BunyanFormatter, Verbosity, vrb_to_level

term_fmt = ColoredFormatter(
    "[%(blue)s%(asctime)s%(reset)s] %(log_color)s%(levelname)-8s%(reset)s %(cyan)s%(name)s %(blue)s%(message)s",  # noqa: E501
    datefmt="%H:%M:%S",
    reset=True,
    log_colors={
        "DEBUG": "cyan",
        "INFO": "green",
        "WARNING": "yellow",
        "ERROR": "red",
        "CRITICAL": "red,bg_white",
    },
    secondary_log_colors={},
    style="%",
)


def setup_logging(
    verbose: Verbosity = None,
    log_file: str | Path | None = None,
    log_file_verbose: Verbosity = None,
):
    """
    Initialize logging and progress bars for a CLI environment.

    Args:
        verbose: whether to include debug output.
        log_file: a log file to write to.
        log_file_verbose: override ``verbose`` for the log file.
    """
    global emgr

    term_level = vrb_to_level(verbose)

    handler = logging.StreamHandler(stream=sys.stderr)
    handler.setFormatter(term_fmt)
    handler.setLevel(term_level)
    root = logging.getLogger()
    root.addHandler(handler)
    if log_file:
        if log_file_verbose is not None:
            file_level = vrb_to_level(log_file_verbose)
        else:
            file_level = term_level
        root.setLevel(min(term_level, file_level))
        fh = logging.FileHandler(log_file, mode="w")
        fh.setLevel(file_level)
        fh.setFormatter(BunyanFormatter())
        root.addHandler(fh)
    else:
        root.setLevel(term_level)

    emgr = Manager(stream=sys.stderr)
    progress_api.set_backend(EnlightenProgressBackend, emgr)



import json
import logging
from logging import Formatter, LogRecord
from typing import TypeAlias

Verbosity: TypeAlias = int | bool | None


def vrb_to_level(verbose: Verbosity) -> int:
    if verbose is None:
        verbose = 0
    if verbose is True:
        verbose = 1

    level = logging.INFO
    if verbose < 0:
        level = logging.WARN
    elif verbose > 0:
        level = logging.DEBUG
    return level


class BunyanFormatter(Formatter):
    def format(self, record: LogRecord) -> str:
        obj = {
            "v": 0,
            "time": self.formatTime(record),
            "level": record.levelno + 10,
            "name": record.name,
            "pid": record.process,
            "msg": record.message,
        }
        return json.dumps(obj)
